// attributes used by GM to give us information about current vertex.
attribute vec3 in_Position;
attribute vec2 in_TextureCoord;

// variable shared with fragment shader with information about texture coordinate.
varying vec2 v_Texcoord;

void main()
{
    // prepare vertex position.
    vec4 pos = vec4(in_Position.xyz, 1.0);
    
    // send vertex texture coordinate to fragment shader.
    v_Texcoord = in_TextureCoord;
    
    // multiply vertex position by WVP matrix so we get screen position of world vertex.
    gl_Position = gm_Matrices[MATRIX_WORLD_VIEW_PROJECTION] * pos;
}

//######################_==_YOYO_SHADER_MARKER_==_######################@~


// texture used for water mask.
uniform sampler2D s_Mask;
// texture used for water normalmap.
uniform sampler2D s_NormalMap;
// normalmap uvs.
uniform vec4 u_NormalMapUvs;
// normalmap offset.
uniform float u_NormalMapOffset;
// normalmap scale.
uniform vec3 u_NormalMapScale;

// texture coordinate variable shared with vertex program.
varying vec2 v_Texcoord;

vec2 uvsGlobalToLocal(vec2 uv, vec4 uvs)
{
    return (uvs.zw - uvs.xy) * fract(uv) + uvs.xy;
}

void main()
{
    // get current pixel color of water mask texture.
    vec4 mask = texture2D(s_Mask, v_Texcoord);

    // get current pixel color of water normalmap texture.
    vec2 ntcoord = v_Texcoord + vec2(u_NormalMapOffset) * u_NormalMapScale.xy;
    vec2 nmcoord = uvsGlobalToLocal(ntcoord, u_NormalMapUvs);
    vec4 normalmap = texture2D(s_NormalMap, nmcoord);
    
    // convert normalmap to distortion vector.
    vec2 distortion = normalmap.xy * vec2(2.0) - vec2(1.0);
    
    // get current distorted pixel color of scene color texture.
    vec4 color = texture2D(gm_BaseTexture, v_Texcoord + distortion * mask.w * u_NormalMapScale.z);
    
    gl_FragColor = color;
}

