// attributes used by GM to give us information about current vertex.
attribute vec3 in_Position;
attribute vec2 in_TextureCoord;

// variable shared with fragment shader with information about texture coordinate.
varying vec2 v_Texcoord;

void main()
{
    // prepare vertex position.
    vec4 pos = vec4(in_Position.xyz, 1.0);
    
    // send vertex texture coordinate to fragment shader.
    v_Texcoord = in_TextureCoord;
    
    // multiply vertex position by WVP matrix so we get screen position of world vertex.
    gl_Position = gm_Matrices[MATRIX_WORLD_VIEW_PROJECTION] * pos;
}

//######################_==_YOYO_SHADER_MARKER_==_######################@~
// texture used for depth.
uniform sampler2D s_Depth;

// offset and altitude parameters.
uniform vec3 u_Offset;
// fog texture frame.
uniform vec4 u_Frame;

// texture coordinate variable shared with vertex program.
varying vec2 v_Texcoord;

vec2 local2global(vec2 c) {
    return (c - u_Frame.xy) / (u_Frame.zw - u_Frame.xy);
}

vec2 global2local(vec2 c) {
    return c * (u_Frame.zw - u_Frame.xy) + u_Frame.xy;
}

void main()
{
    // get current pixel color of fog texture.
    vec2 global = local2global(v_Texcoord);
    vec2 local = global2local(fract(global + u_Offset.xy));
    vec4 color = texture2D(gm_BaseTexture, local);

    // get current pixel color of depth texture.
    float depth = texture2D(s_Depth, global).r;
    
    // show fog only where depth is low.
    float f = u_Offset.z - depth;
    gl_FragColor = vec4(color.rgb, f * color.a);
}

